# Packages and quick start guide

GLB Director's components are packaged and ready to use along with a compatible DPDK version for Debian Jessie and Debian Stretch.

![L4/L7 load balancer design](../images/glb-component-overview.png)

## Installing the apt sources

An unofficial DPDK release apt source is provided which provides a compatible DPDK version to use with GLB director - currently DPDK `17.11.1-6`:
```
curl -s https://packagecloud.io/install/repositories/github/unofficial-dpdk-stable/script.deb.sh | sudo bash
```

The GLB packages themselves are also available as an apt source:
```
curl -s https://packagecloud.io/install/repositories/github/glb-director/script.deb.sh | sudo bash
```

## Director tier: `glb-director`

The main `glb-director` DPDK application takes packets destined to a GLB IP and encapsulates them over a tunnel to one of the configured backend proxy servers.

Installation on Debian:
```
sudo apt install glb-director
```

The following configuration files need to be adjusted, see [glb-director configuration](./glb-director-configuration.md) and [GLB forwarding table configuration](./forwarding-table-config.md) for more information:
 * `/etc/default/glb-director` - Configuration around the DPDK EAL runtime and other similar configuration options, used by systemd.
 * `/etc/glb/director.conf` - The default location for the configuration of how the director processes packets on CPU cores and on ports/NICs.
 * `/etc/glb/forwarding_table.json` - The default location for the forwarding table when healthchecking is not used.

The glb-director runs with systemd under the `glb-director.service` unit.

## Director tier: `glb-healthcheck`

The `glb-healthcheck` application runs tunnel and HTTP healthchecks on a source [GLB forwarding table](./forwarding-table-config.md), writing out a new table that `glb-director` can consume that takes into account health state of backend proxy servers.

Installation on Debian:
```
sudo apt install glb-healthcheck
```

To configure the healthchecker to work with `glb-director`, the `/etc/default/glb-director` will need to be updated to reflect the `/etc/glb/forwarding_table.checked.bin` version.

The following files are relevant to configuring the healthchecker itself, see [GLB forwarding table configuration](./forwarding-table-config.md) for more information:
 * `/etc/glb/healthcheck.conf` - The default location for the configuration of how the healthchecker performs its functions.
 * `/etc/glb/forwarding_table.src.json` - The default location for the forwarding table when healthchecking is used.
 * `/etc/glb/forwarding_table.checked.json` - Auto-generated by `glb-healthcheck`.

The glb-healthcheck runs with systemd under the `glb-healthcheck.service` unit.

## Proxy tier: `glb-redirect-iptables-dkms`

The `glb-redirect-iptables-dkms` package uses [DKMS](https://en.wikipedia.org/wiki/Dynamic_Kernel_Module_Support) to create and maintain an iptables module for GLB purposes that works on the currently running kernel. 

Installation on Debian:
```
sudo apt install glb-redirect-iptables-dkms
```

This package provides the `GLB-REDIRECT` iptables target, see [Backend Proxy Setup](./backend-proxy-setup.md) for more information on configuring proxy tier servers including usage of this iptables module.
